@@ -5,18 +5,22 @@ import os
 import re
 import string
 import struct
+import zlib
+
+from pathlib import Path
 
 import vdf
 
 __all__ = (
     "COMMON_STEAM_DIRS", "SteamApp", "find_steam_path",
     "find_steam_proton_app", "find_proton_app", "find_steam_runtime_path",
-    "get_steam_lib_paths", "get_steam_apps", "get_custom_proton_installations"
+    "find_appid_proton_prefix", "get_steam_lib_paths", "get_steam_apps",
+    "get_custom_proton_installations"
 )
 
 COMMON_STEAM_DIRS = [
-    os.path.join(os.environ.get("HOME"), ".steam", "steam"),
-    os.path.join(os.environ.get("HOME"), ".local", "share", "Steam")
+    os.path.join(".steam", "steam"),
+    os.path.join(".local", "share", "Steam")
 ]
 
 logger = logging.getLogger("protontricks")
@@ -24,7 +28,9 @@ logger = logging.getLogger("protontricks")
 
 class SteamApp(object):
     """
-    SteamApp represents an installed Steam app
+    SteamApp represents an installed Steam app or whatever is close enough to
+    one (eg. a custom Proton installation or a Windows shortcut with its own
+    Proton prefix)
     """
     __slots__ = ("appid", "name", "prefix_path", "install_path")
 
@@ -86,7 +92,7 @@ class SteamApp(object):
         return os.path.exists(os.path.join(self.install_path, "proton"))
 
     @classmethod
-    def from_appmanifest(cls, path):
+    def from_appmanifest(cls, path, steam_lib_paths):
         """
         Parse appmanifest_X.acf file containing Steam app installation metadata
         and return a SteamApp object
@@ -124,8 +130,11 @@ class SteamApp(object):
         app_state = {k.lower(): v for k, v in app_state.items()}
         appid = int(app_state["appid"])
         name = app_state["name"]
-        prefix_path = os.path.join(
-            os.path.split(path)[0], "compatdata", str(appid), "pfx")
+
+        # Proton prefix may exist on a different library
+        prefix_path = find_appid_proton_prefix(
+            appid=appid, steam_lib_paths=steam_lib_paths
+        )
 
         install_path = os.path.join(
             os.path.split(path)[0], "common", app_state["installdir"])
@@ -156,6 +165,9 @@ def find_steam_path():
             or os.path.isdir(os.path.join(path, "SteamApps"))
         )
 
+    def has_runtime_dir(path):
+        return os.path.isdir(os.path.join(path, "ubuntu12_32"))
+
     # as far as @admalledd can tell,
     # this should always be correct for the tools root:
     steam_root = os.path.join(os.path.expanduser("~"), ".steam", "root")
@@ -166,14 +178,12 @@ def find_steam_path():
 
     if os.environ.get("STEAM_DIR"):
         steam_path = os.environ.get("STEAM_DIR")
-        if has_steamapps_dir(steam_path):
+        if has_steamapps_dir(steam_path) and has_runtime_dir(steam_path):
             logger.info(
-                "Found a valid Steam installation at {}.".format(steam_path)
+                "Found a valid Steam installation at %s.", steam_path
             )
 
-            if not steam_root:
-                steam_root = steam_path
-            return steam_path, steam_root
+            return steam_path, steam_path
 
         logger.error(
             "$STEAM_DIR was provided but didn't point to a valid Steam "
@@ -183,6 +193,8 @@ def find_steam_path():
         return None, None
 
     for steam_path in COMMON_STEAM_DIRS:
+        # The common Steam directories are found inside the home directory
+        steam_path = str(Path.home() / steam_path)
         if has_steamapps_dir(steam_path):
             logger.info(
                 "Found Steam directory at {}. You can also define Steam "
@@ -374,7 +386,7 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
     except KeyError:
         compat_tool_mapping = {}
 
-    name = None
+    compat_tool_name = None
 
     # The name of potential names in order of priority
     potential_names = [
@@ -399,7 +411,7 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
 
     # Let's try option 1 first
     try:
-        app = next(app for app in steam_apps if app.name == name)
+        app = next(app for app in steam_apps if app.name == compat_tool_name)
         logger.info(
             "Found active custom Proton installation: {}".format(app.name)
         )
@@ -427,6 +439,53 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
         return None
 
 
+def find_appid_proton_prefix(appid, steam_lib_paths):
+    """
+    Find the Proton prefix for the app by its App ID
+
+    Proton prefix and the game installation itself can exist on different
+    Steam libraries, making a search necessary
+    """
+    def get_prefix_modify_time(prefix_path):
+        """
+        Get the prefix modification time for sorting purposes.
+        The newest modification time corresponds to the most recently
+        used Proton prefix
+        """
+        try:
+            # 'pfx.lock' is modified on game launch
+            return os.stat(
+                os.path.join(prefix_path, "..", "pfx.lock")
+            ).st_mtime
+        except FileNotFoundError:
+            return 0
+
+    candidates = []
+
+    for path in steam_lib_paths:
+        # 'steamapps' portion of the path can also be 'SteamApps'
+        for steamapps_part in ("steamapps", "SteamApps"):
+            prefix_path = os.path.join(
+                path, steamapps_part, "compatdata", str(appid), "pfx"
+            )
+            if os.path.isdir(prefix_path):
+                candidates.append(prefix_path)
+
+    if len(candidates) > 1:
+        # If we have more than one possible prefix path, use the one
+        # with the most recent modification date
+        logger.info(
+            "Multiple compatdata directories found for app %s", appid
+        )
+        candidates.sort(key=get_prefix_modify_time)
+        candidates.reverse()
+
+    if candidates:
+        return candidates[0]
+
+    return None
+
+
 def find_proton_app(steam_path, steam_apps, appid=None):
     """
     Find the Proton app, using either $PROTON_VERSION or the one
@@ -552,14 +611,124 @@ def get_custom_proton_installations(steam_root):
     return custom_proton_apps
 
 
-def get_steam_apps(steam_root, steam_lib_dirs):
+def find_current_steamid3(steam_path):
+    def to_steamid3(steamid64):
+        """Convert a SteamID64 into the SteamID3 format"""
+        return int(steamid64) & 0xffffffff
+
+    loginusers_path = os.path.join(steam_path, "config", "loginusers.vdf")
+    try:
+        with open(loginusers_path, "r") as f:
+            content = f.read()
+            vdf_data = vdf.loads(content)
+    except IOError:
+        return None
+
+    users = [
+        {
+            "steamid3": to_steamid3(user_id),
+            "account_name": user_data["AccountName"],
+            "timestamp": user_data.get("Timestamp", 0)
+        }
+        for user_id, user_data in vdf_data["users"].items()
+    ]
+
+    # Return the user with the highest timestamp, as that's likely to be the
+    # currently logged-in user
+    if users:
+        user = max(users, key=lambda u: u["timestamp"])
+        logger.info(
+            "Currently logged-in Steam user: %s", user["account_name"]
+        )
+        return user["steamid3"]
+
+    return None
+
+
+def get_appid_from_shortcut(target, name):
+    """
+    Get the identifier used for the Proton prefix from a shortcut's
+    target and name
+    """
+    # First, calculate the screenshot ID Steam uses for shortcuts
+    data = b"".join([
+        target.encode("utf-8"),
+        name.encode("utf-8")
+    ])
+    result = zlib.crc32(data) & 0xffffffff
+    result = result | 0x80000000
+    result = (result << 32) | 0x02000000
+
+    # Derive the prefix ID from the screenshot ID
+    return result >> 32
+
+
+def get_custom_windows_shortcuts(steam_path):
+    """
+    Get a list of custom shortcuts for Windows applications as a list
+    of SteamApp objects
+    """
+    # Get the Steam ID3 for the currently logged-in user
+    steamid3 = find_current_steamid3(steam_path)
+
+    shortcuts_path = os.path.join(
+        steam_path, "userdata", str(steamid3), "config", "shortcuts.vdf"
+    )
+
+    try:
+        with open(shortcuts_path, "rb") as f:
+            content = f.read()
+            vdf_data = vdf.binary_loads(content)
+    except IOError:
+        logger.info(
+            "Couldn't find custom shortcuts. Maybe none have been created yet?"
+        )
+        return []
+
+    steam_apps = []
+
+    for shortcut_id, shortcut_data in vdf_data["shortcuts"].items():
+        # The "exe" field can also be "Exe". Account for this by making
+        # all field names lowercase
+        shortcut_data = {k.lower(): v for k, v in shortcut_data.items()}
+        shortcut_id = int(shortcut_id)
+
+        appid = get_appid_from_shortcut(
+            target=shortcut_data["exe"], name=shortcut_data["appname"]
+        )
+
+        prefix_path = os.path.join(
+            steam_path, "steamapps", "compatdata", str(appid), "pfx"
+        )
+        install_path = shortcut_data["startdir"].strip('"')
+
+        if not os.path.isdir(prefix_path):
+            continue
+
+        steam_apps.append(
+            SteamApp(
+                appid=appid,
+                name="Non-Steam shortcut: {}".format(shortcut_data["appname"]),
+                prefix_path=prefix_path, install_path=install_path
+            )
+        )
+
+    logger.info(
+        "Found %d Steam shortcuts running under Proton", len(steam_apps)
+    )
+
+    return steam_apps
+
+
+def get_steam_apps(steam_root, steam_path, steam_lib_paths):
     """
     Find all the installed Steam apps and return them as a list of SteamApp
     objects
     """
     steam_apps = []
 
-    for path in steam_lib_dirs:
+    for path in steam_lib_paths:
+        appmanifest_paths = []
         if os.path.isdir(os.path.join(path, "steamapps")):
             appmanifest_paths = glob.glob(
                 os.path.join(path, "steamapps", "appmanifest_*.acf")
@@ -569,13 +738,16 @@ def get_steam_apps(steam_root, steam_lib_dirs):
                 os.path.join(path, "SteamApps", "appmanifest_*.acf")
             )
 
-        for path in appmanifest_paths:
-            steam_app = SteamApp.from_appmanifest(path)
+        for manifest_path in appmanifest_paths:
+            steam_app = SteamApp.from_appmanifest(
+                manifest_path, steam_lib_paths=steam_lib_paths
+            )
             if steam_app:
                 steam_apps.append(steam_app)
 
-    # Get the custom Proton installations as well
+    # Get the custom Proton installations and non-Steam shortcuts as well
     steam_apps += get_custom_proton_installations(steam_root=steam_root)
+    steam_apps += get_custom_windows_shortcuts(steam_path=steam_path)
 
     # Exclude games that haven't been launched yet
     steam_apps = [
